/**
 * synopsis: Generates an Fluxbox menu segment based on an XDG menu structure
 * purpose: The purpose of this utility is to keep the Fluxbox menu
 *          synchronized with the XDG configuration
 * usage: fluxbox-xdgmenu <xdgMenuFile>
 * original author: Raul Suarez (rarsa at yahoo dot com)
 * fluxbox version by Nathan Fisher (nfisher at grafpup dot com)
 * Licence: GPL
 *
 * Dependencies:
 *   gnome-menus 2.12
 *   glib
 */

#include <glib.h>
#include <glib/gprintf.h>
#include <gmenu-tree.h>


/*=============================================================================
 * Declarations
 */
static void show_help();
static void write_header();
static void process_directory(GMenuTreeDirectory *directory, int isRoot);
static void process_entry(GMenuTreeEntry *entry);
static void process_separator(GMenuTreeSeparator *entry);


/*=============================================================================
 * Main Function
 */
int
main (int argc, char **argv)
{

  if ((argc != 2)
      || (strcmp (argv[1], "-h") == 0)
      || (strcmp (argv[1], "-help") == 0)
      || (strcmp (argv[1], "--help") == 0))
  {
    show_help();
    return (1);
  }

  GMenuTree *menuTree = gmenu_tree_lookup (argv[1],  GMENU_TREE_FLAGS_NONE);

  GMenuTreeDirectory *rootDirectory = gmenu_tree_get_root_directory(menuTree);

  /* write_header(); */

  process_directory(rootDirectory, 1);

  gmenu_tree_item_unref (rootDirectory);

  return 0;

}

/*=============================================================================
 * Shows usage parameters
 */
void
show_help()
{
  printf ("Creates a fluxbox menu from an XDG menu structure.\n");
  printf ("Usage:\n");
  printf ("  fluxbox-xdgmenu xdgmenufile\n");
  printf ("\n");
  printf ("  xdgmenufile: Fully qualified path to the XDG menu file. This is a mandatory parameter\n");
  printf ("\n");
  printf ("For example:\n");
  printf ("  fluxbox-xdgmenu \"/etc/xdg/menus/applications.menu\"\n\n");
  printf ("\n");
}

/*=============================================================================
 * This function writes the header */

/* void
 write_header()
{
 printf ("[begin] (Fluxbox)\n");
}
*/
/*=============================================================================
 * This function processes a directory entry and all it's child nodes
 */
void
process_directory(GMenuTreeDirectory *directory, int isRoot)
{
	int hasSeparator = 0;

/*  if (!isRoot)
  { */
    g_printf("[submenu] (%s) \n",
              gmenu_tree_directory_get_name(directory));
 /* } */

  GMenuTreeItemType entryType;

  GSList *entryList = gmenu_tree_directory_get_contents (directory);

  GSList *l;

  for (l = entryList; l; l = l->next)
  {

    GMenuTreeItem *item = l->data;

    entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

	switch (entryType)
	{
		case GMENU_TREE_ITEM_DIRECTORY:
		  if (hasSeparator)
		  {
				process_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
			process_directory(GMENU_TREE_DIRECTORY(item), 0);
			break;
		case GMENU_TREE_ITEM_ENTRY:
		  if (hasSeparator)
		  {
				process_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
			process_entry(GMENU_TREE_ENTRY(item));
			break;
		case GMENU_TREE_ITEM_SEPARATOR:
			hasSeparator = 1;
			break;
	}

    gmenu_tree_item_unref (item);

  }

  g_slist_free (entryList);

 /* if (!isRoot)
  { */
    g_printf("[end] \n");
/* } */
}

/*=============================================================================
 * This function adds an application entry
 */
void
process_entry(GMenuTreeEntry *entry)
{

  g_printf("[exec] (%s) {%s}\n",
            gmenu_tree_entry_get_name(entry),
            gmenu_tree_entry_get_exec (entry));
}

/*=============================================================================
 * This function adds a separator
 */
void
process_separator(GMenuTreeSeparator *entry)
{

  g_printf("[separator]\n");
}

/*=============================================================================
 */
